var SpecimenTubeReg = function()
{
	var streg = {};
	var debug = 0;
	
	var caseInfo;
	
	var nofTubesIsValid = false;
	var samplingDateIsValid = true;
	var rnaLaterDateIsValid = true;
	var lateralityIsValid = true;
	var biopsyTypeIsValid = true;

	var wellIsValid = [];
	
	var restartTimer;
	var restartAt;
	
	// Page initialization
	streg.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', streg.validateStep1);
		Events.addEventHandler('caseName', 'keypress', Wizard.goNextOnTabOrEnter);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', streg.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', streg.validateStep2);
		Events.addEventHandler('nofTubes', 'keypress', Events.integerOnly);
		Events.addEventHandler('nofTubes', 'blur', streg.nofTubesOnChange);
		Events.addEventHandler('arrivalDate', 'blur', Wizard.validateDate);
		Events.addEventHandler('samplingDate', 'blur', streg.samplingDateOnChange);
		Events.addEventHandler('samplingTime', 'blur', streg.samplingDateOnChange);
		Events.addEventHandler('rnaLaterDate', 'blur', streg.rnaLaterDateOnChange);
		Events.addEventHandler('rnaLaterTime', 'blur', streg.rnaLaterDateOnChange);
		
		Events.addEventHandler('lateralityLEFT', 'click', streg.lateralityOnChange);
		Events.addEventHandler('lateralityRIGHT', 'click', streg.lateralityOnChange);
		Events.addEventHandler('lateralityU', 'click', streg.lateralityOnChange);
		Events.addEventHandler('biopsyType', 'change', streg.biopsyTypeOnChange);
		Events.addEventHandler('otherPathNote', 'keypress', Wizard.goNextOnTab);
		
		// Step 3
		Events.addEventHandler('step-3', 'wizard-initialize', streg.initializeStep3);
		Events.addEventHandler('step-3', 'wizard-validate', streg.validateStep3);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		Buttons.addClickHandler('gostop', streg.stopAutoRestart);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', streg.submit);

		Doc.show('step-1');
		Doc.show('gonext');
	}

	streg.validateStep1 = function(event)
	{
		var frm = document.forms['reggie'];
		
		var caseName = frm.caseName.value;
		var caseIsValid = false;
		Wizard.setInputStatus('caseName');
		if (caseName == '')
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Missing');
			frm.caseName.focus();
		}
		else if (!Reggie.isValidCaseName(caseName, true))  // Allow C/D suffix to be used
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Only 7-digits name + optional \'C\' or \'D\' is allowed.');
			frm.caseName.focus();
		}
		else
		{
			Wizard.setInputStatus('caseName', 'valid');
			caseIsValid = true;
		}
		
		// If not valid, we prevent the wizard from moving to the next step
		if (!caseIsValid) event.preventDefault();
	}
	
	streg.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];	
		var caseName = frm.caseName.value;
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfo';
		url += '&caseName=' + encodeURIComponent(caseName);
		
		Wizard.showLoadingAnimation('Loading information about case...');
		Wizard.asyncJsonRequest(url, streg.caseInfoLoaded);
	}
	
	
	streg.caseInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];	
		caseInfo = response.caseInfo;
		
		Wizard.setCurrentStep(2);
		Doc.show('gocancel');
		Doc.show('gonext');
		
		frm.arrivalDate.value = Dates.formatDate(new Date(), 'yyyyMMdd');

		frm.nofTubes.focus();
		frm.nofTubes.setSelectionRange(0,1);
		
		// Select 'Laterality' if we have that information since before
		if (caseInfo.laterality)
		{
			Forms.checkRadio(frm.laterality, caseInfo.laterality);
			streg.lateralityOnChange();
			
			if (caseInfo.laterality == 'LEFT') Doc.hide('lateralityRight');
			if (caseInfo.laterality == 'RIGHT') Doc.hide('lateralityLeft');
			Doc.hide('lateralityUnknown');
			Doc.element('lateralityFromCase').innerHTML = '(From existing case: ' + Strings.encodeTags(caseInfo.name) + ')';
			Doc.show('lateralityFromCase');
		}
		
		
		var caseName = frm.caseName.value;
		if (caseName.length > 7)
		{
			var suffix = caseName.substring(7, 8);
			if (suffix == 'C')
			{
				// Select 'SpecimenCoreBiopsy' if case name has 'C' suffix
				Forms.selectListOption(frm.biopsyType, 'SpecimenCoreBiopsy');
			}
			else if (suffix == 'D')
			{
				// Select 'SpecimenCoreBiopsy2nd' if case name has 'D' suffix
				Forms.selectListOption(frm.biopsyType, 'SpecimenCoreBiopsy2nd');
			}
			streg.biopsyTypeOnChange();
		}
	}
	
	streg.nofTubesOnChange = function()
	{
		nofTubesIsValid = false;

		var frm = document.forms['reggie'];
		var nofTubes = frm.nofTubes.value;
		
		if (nofTubes == '')
		{
			Wizard.setInputStatus('nofTubes', 'invalid', 'Missing');
			return;
		}
		if (parseInt(nofTubes) < 1)
		{
			Wizard.setInputStatus('nofTubes', 'invalid', 'Must be at least 1');
			return;
		}
		if (parseInt(nofTubes) > 9)
		{
			Wizard.setInputStatus('nofTubes', 'invalid', 'Must not be greater than 9');
			return;		
		}
		Wizard.setInputStatus('nofTubes', 'valid');
		nofTubesIsValid = true; 
	}

	
	streg.samplingDateOnChange = function()
	{
		var frm = document.forms['reggie'];
		
		samplingDateIsValid = false;
		Wizard.setInputStatus('samplingDate');
		
		var samplingDate = frm.samplingDate.value;
		var samplingTime = frm.samplingTime.value;
		
		if (samplingDate != '' || samplingTime != '')
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			samplingDate = Reggie.autoFillDate(samplingDate);
			frm.samplingDate.value = samplingDate;

			samplingTime = Reggie.autoFillTime(samplingTime);
			frm.samplingTime.value = samplingTime;
			
			var d = Dates.parseString(samplingDate, 'yyyyMMdd');
			if (d == null)
			{
				Wizard.setInputStatus('samplingDate', 'invalid', 'Not a valid date');
				return;
			}
			
			var today = new Date();
			if (d.getTime() > today.getTime())
			{
				Wizard.setInputStatus('samplingDate', 'invalid', 'Future date is not valid');
				return;
			}
			if (frm.rnaLaterDate.value == '') frm.rnaLaterDate.value = samplingDate;
			
			if (!Dates.isDate(samplingDate + ' ' + samplingTime, 'yyyyMMdd HHmm'))
			{
				if (samplingDate != '') 
				{
					Wizard.setInputStatus('samplingDate', 'invalid', 'Not a valid time (if time unknown, also leave date field blank)');
				}
				return;
			}
			Wizard.setInputStatus('samplingDate', 'valid');
		}
		samplingDateIsValid = true;
		streg.rnaLaterDateOnChange();
	}

	
	streg.rnaLaterDateOnChange = function()
	{
		var frm = document.forms['reggie'];

		rnaLaterDateIsValid = false;
		Wizard.setInputStatus('rnaLaterDate');
		
		var rnaLaterDate = frm.rnaLaterDate.value;
		var rnaLaterTime = frm.rnaLaterTime.value;
		var rnaLaterTimestamp;
		
		if (rnaLaterDate != '' || rnaLaterTime != '')
		{
			// Auto-fill the date if it's not given in yyyyMMdd
			rnaLaterDate = Reggie.autoFillDate(rnaLaterDate);
			frm.rnaLaterDate.value = rnaLaterDate;

			rnaLaterTime = Reggie.autoFillTime(rnaLaterTime);
			frm.rnaLaterTime.value = rnaLaterTime;
			
			var d = Dates.parseString(rnaLaterDate, 'yyyyMMdd');
			if (d == null)
			{
				Wizard.setInputStatus('rnaLaterDate', 'invalid', 'Not a valid date');
				return;
			}
			var today = new Date();
			if (d.getTime() > today.getTime())
			{
				Wizard.setInputStatus('rnaLaterDate', 'invalid', 'Future date is not valid');
				return;
			}
			
			var rnaLaterTimestamp = Dates.parseString(rnaLaterDate + ' ' + rnaLaterTime, 'yyyyMMdd HHmm');
			if (rnaLaterTimestamp == null)
			{
				if (rnaLaterDate != '') 
				{
					Wizard.setInputStatus('rnaLaterDate', 'invalid', 'Not a valid time (if time unknown, also leave date field blank)');
				}
				return;
			}
			if (samplingDateIsValid && rnaLaterTimestamp)
			{
				var samplingTimestamp = Dates.parseString(frm.samplingDate.value + ' ' + frm.samplingTime.value, 'yyyyMMdd Hmm');			
				if (samplingTimestamp)
				{
					if (rnaLaterTimestamp.getDate() != samplingTimestamp.getDate())
					{
						Wizard.setInputStatus('rnaLaterDate', 'invalid', 'Sampling and RNA later dates are different');
						return;
					}
					if (rnaLaterTimestamp.getTime() < samplingTimestamp.getTime())
					{
						Wizard.setInputStatus('rnaLaterDate', 'invalid', 'RNA later time must be later than Sampling time');
						return;
					}
				}
				
			}
			Wizard.setInputStatus('rnaLaterDate', 'valid');
		}
			
		rnaLaterDateIsValid = true;
	}

	
	streg.lateralityOnChange = function()
	{	
		lateralityIsValid = false;
		
		// Check selected laterality against specimen tubes
		var frm = document.forms['reggie'];
		var selectedLaterality = Forms.getCheckedRadio(frm.laterality);
		
		// No laterality/case selected
		if (selectedLaterality == null) 
		{
			Wizard.setInputStatus('laterality', 'invalid', 'Not selected');
			return;
		}
		
		lateralityIsValid = true;

		if (caseInfo.laterality && caseInfo.laterality != selectedLaterality.value)
		{
			Wizard.setInputStatus('laterality', 'warning', 'Not same laterality as existing case/specimen');	
			return;
		}
		
		Wizard.setInputStatus('laterality', 'valid');
	}

	streg.biopsyTypeOnChange = function()
	{
		var frm = document.forms['reggie'];
		var selectedBiopsyType = frm.biopsyType.value;
		biopsyTypeIsValid = false;
		
		// Check 'SpecimenCoreBiopsy' if case name has 'C' suffix
		var caseName = frm.caseName.value;
		var suffix = caseName.length > 7 ? caseName.substring(7, 8) : null;
		if (suffix == 'C' && selectedBiopsyType != 'SpecimenCoreBiopsy')
		{
			Wizard.setInputStatus('biopsyType', 'invalid', 'Case name with \'C\' suffix is always SpecimenCoreBiopsy');
			return;
		}
		if (suffix == 'D' && selectedBiopsyType != 'SpecimenCoreBiopsy2nd')
		{
			Wizard.setInputStatus('biopsyType', 'invalid', 'Case name with \'D\' suffix is always SpecimenCoreBiopsy2nd');
			return;
		}
		
		biopsyTypeIsValid = true;
		Wizard.setInputStatus('biopsyType', 'valid');
	}

	
	streg.validateStep2 = function(event)
	{
		var valid = true;
		var frm = document.forms['reggie'];	

		valid &= nofTubesIsValid;
		valid &= Wizard.isValid('arrivalDate');
		valid &= samplingDateIsValid;
		valid &= rnaLaterDateIsValid;
		valid &= lateralityIsValid;
		valid &= biopsyTypeIsValid;
		
		if (!valid) event.preventDefault();
	}

	streg.initializeStep3 = function()
	{
		var frm = document.forms['reggie'];
		var nofTubes = parseInt(frm.nofTubes.value);
		if (nofTubes > 2) Doc.addClass('step-2', 'auto-hide');
		
		var html = '';
		for (var i=0; i < nofTubes; i++)
		{
			var tubeNo = i+caseInfo.specimenFirstIndex;
			html += '<tbody id="specimenSection.'+i+'">';
			html += '<tr class="section-header">';
			html += '<td colspan="4">Specimen Tube '+Strings.encodeTags(caseInfo.originalName)+'.'+tubeNo+'</td>';
			html += '</tr>';

			html += '<tr>';
			html += '<td class="subprompt yellow-tag"><label for="yellowLabel.'+i+'">Yellow label</label></td>';
			html += '<td class="input">';
			html += '<input type="checkbox" name="yellowLabel.'+i+'" id="yellowLabel.'+i+'"></td>';
			html += '<td class="status"></td>';
			html += '<td class="help"></td>';		
			html += '</tr>';
			
			html += '<tr>'
			html += '<td class="subprompt">Specimen type</td>';
			html += '<td class="input">';
			html += '<select name="specimenType.'+i+'" id="specimenType.'+i+'">';
			html += '<option selected value="">unknown';
			html += '<option value="Primary">Primary';
			html += '<option value="LymphnodeRegional">LymphnodeRegional';				
			html += '<option value="MetastasisDistant">MetastasisDistant';
			html += '<option value="RecurrenceLocal">RecurrenceLocal';
			html += '<option value="RecurrenceRegional">RecurrenceRegional';
			html += '<option value="Normal">Normal';
			html += '<option value="Uncertain">Uncertain';
			html += '</select>';
			html += '</td>';
			html += '<td class="status" id="specimenType.'+i+'.status"></td>';
			html += '<td class="help"><span id="specimenType.'+i+'.message" class="message"></span></td>';		
			html += '</tr>';
			
			html += '<tr>';
			html += '<td class="subprompt">Box</td>';
			html += '<td class="input">';
			html += '<input type="text" name="box.'+i+'" id="box.'+i+'" style="width: 12em;" maxlength="12"></td>';
			html += '<td class="status" id="box.'+i+'.status"></td>';
			html += '<td class="help"><span id="box.'+i+'.message" class="message"></span>Box-number where the specimen tube is located in.</td>';		
			html += '</tr>';
			
			html += '<tr>';
			html += '<td class="subprompt">Row</td>';
			html += '<td class="input">';
			html += '<input type="text" name="row.'+i+'" id="row.'+i+'" style="text-transform:uppercase; width: 3em;" maxlength="2">';
			html += '&nbsp;Column<input type="text" name="column.'+i+'" id="column.'+i+'" style="width: 3em;" maxlength="2">';
			html += '</td>';
			html += '<td class="status" id="position.'+i+'.status"></td>';
			html += '<td class="help"><span id="position.'+i+'.message" class="message"></span></td>';		
			html += '</tr>';

			html += '<tr class="align-top">';
			html += '<td class="subprompt">Operator delivery comment</td>';
			html += '<td class="input"><textarea rows="3" name="comment.'+i+'" id="comment.'+i+'"></textarea></td>';
			html += '<td class="status" id="comment.'+i+'.status"></td>';
			html += '<td class="help"><span id="comment.'+i+'.message" class="message"></span>In this field, enter information specific for specimen tube ' +tubeNo+'.</td>';		
			html += '</tr>';
			
			html += '</tbody>';
		}
		Doc.element('specimen-tubes').innerHTML = html;

		Wizard.setCurrentStep(3);
		Doc.show('goregister');
		Doc.show('gocancel');
		frm['specimenType.0'].focus();
		
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=FindStoragePositions';	
		url += '&nofTubes='+frm.nofTubes.value;
		
		Wizard.showLoadingAnimation('Looking for empty box positions...');
		Wizard.asyncJsonRequest(url, streg.storagePositionsLoaded);
	}
	
	streg.storagePositionsLoaded = function(response)
	{
		var freeWells = response.wells;
		var frm = document.forms['reggie'];
		var nofTubes = parseInt(frm.nofTubes.value);

		for (var i = 0; i < nofTubes; i++)
		{
			var well = freeWells[i];
			frm['box.'+i].value = well.bioPlate.name;
			frm['row.'+i].value = Reggie.wellToAlpha(well.row);
			frm['column.'+i].value = well.column+1;
			
			// Move focus to next field
			Events.addEventHandler('specimenType.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'box.'+i });
			Events.addEventHandler('box.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'row.'+i });
			Events.addEventHandler('row.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'column.'+i });
			Events.addEventHandler('column.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'comment.'+i });
			
			// Form validation
			Events.addEventHandler('column.'+i, 'keypress', Events.integerOnly);
			Events.addEventHandler('box.'+i, 'change', streg.wellOnChange, { 'index': i });
			Events.addEventHandler('row.'+i, 'change', streg.wellOnChange, { 'index': i });
			Events.addEventHandler('column.'+i, 'change', streg.wellOnChange, { 'index': i });
			Events.addEventHandler('box.'+i, 'keyup', streg.boxOnKeyUp);
			
			// Yellow label
			Events.addEventHandler('yellowLabel.'+i, 'change', streg.yellowLabelOnChange, { 'index': i });
			
			wellIsValid[i] = true;
			Wizard.setInputStatus('position.'+i, 'valid');
			Wizard.setInputStatus('box.'+i, 'valid');
		}
	}
	
	// Update class on specimen section
	streg.yellowLabelOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var index = Data.int(event.currentTarget, 'index');
		Doc.addOrRemoveClass('specimenSection.'+index, 'yellow-label', event.currentTarget.checked);
	}
	
	streg.boxOnKeyUp = function(event)
	{
		var frm = document.forms['reggie'];
		
		var index = Data.int(event.currentTarget, 'index');

		var box = frm['box.'+index].value;
		if (box.indexOf('Sp') != 0)
		{
			frm['box.'+index].value = 'Sp'+box;
		}
	}

	
	streg.wellOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var target = event.currentTarget;

		var index = Data.int(target, 'index');
		var box = frm['box.'+index].value;
		var row = frm['row.'+index].value;
		var column = frm['column.'+index].value;
		
		wellIsValid[index] = false;
		Wizard.setInputStatus('box.'+index);
		Wizard.setInputStatus('position.'+index);
		
		if (box == '') 
		{
			Wizard.setInputStatus('box.'+index, 'invalid', 'Missing box');
			return;
		}
		
		if (row == '')
		{
			Wizard.setInputStatus('position.'+index, 'invalid', 'Missing row');
			return;
		}

		if (column == '')
		{
			Wizard.setInputStatus('position.'+index, 'invalid', 'Missing column');
			return;
		}
		
		// Check for duplicates
		var nofTubes = parseInt(frm.nofTubes.value);
		for (var i=0; i < nofTubes; i++)
		{		
			if (i != index)
			{
				if (box == frm.elements['box.'+i].value && row == frm['row.'+i].value && column == frm['column.'+i].value)
				{
					Wizard.setInputStatus('position.'+index, 'invalid', 'The position is already used once in this wizard');				
					return;
				}
			}
		}

		Wizard.setInputStatus('position.'+index, 'checking', 'Checking...');
		
		// Seems to be ok -- check if used by another sample
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=ValidateWell';
		url += '&box=' + encodeURIComponent(box);	
		url += '&row=' + encodeURIComponent(row);
		url += '&column=' + encodeURIComponent(column);
		
		Wizard.asyncJsonRequest(url, function(response) { streg.onBoxValidated(response, index) } );
	}
	
	streg.onBoxValidated = function(response, index)
	{
		Wizard.setInputStatus('position.'+index);
		if (response.message)
		{
			Wizard.setInputStatus(response.box ? 'box.'+index : 'position.'+index, 'invalid', Strings.encodeTags(response.message));
			return;
		}
		
		wellIsValid[index] = true;
		Wizard.setInputStatus('box.'+index, 'valid');
		Wizard.setInputStatus('position.'+index, 'valid');
	}

	
	streg.validateStep3 = function(event)
	{
		var valid = false;

		for (var i = 0; i < wellIsValid.length; i++)
		{
			if (!wellIsValid[i])
			{
				event.preventDefault();
				return;
			}
		}
	}

	
	streg.submit = function()
	{
		var frm = document.forms['reggie'];
		
		caseInfo.laterality = Forms.getCheckedRadio(frm.laterality).value;
		caseInfo.samplingDate = Strings.trim(frm.samplingDate.value + ' ' + frm.samplingTime.value);
		caseInfo.rnaLaterDate = Strings.trim(frm.rnaLaterDate.value + ' ' + frm.rnaLaterTime.value);
		caseInfo.otherPathNote = frm.otherPathNote.value;
		caseInfo.arrivalDate = Strings.trim(frm.arrivalDate.value);
		
		var nofTubes = parseInt(frm.nofTubes.value);
		caseInfo.specimen = [];
		for (var i = 0; i < nofTubes; i++)
		{
			var specimenTube ={};
			specimenTube.name = caseInfo.originalName+'.'+(i+caseInfo.specimenFirstIndex);

			specimenTube.box = frm['box.'+i].value;
			specimenTube.row = frm['row.'+i].value;
			specimenTube.column = frm['column.'+i].value;
			specimenTube.comment = frm['comment.'+i].value;
			specimenTube.specimenType = frm['specimenType.'+i].value;
			specimenTube.yellowLabel = frm['yellowLabel.'+i].checked;
			specimenTube.biopsyType = frm.biopsyType.value;
			caseInfo.specimen[i] = specimenTube;
		}

		var submitInfo = {};
		submitInfo.caseInfo = caseInfo;

		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=CreateSpecimenTubes';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, streg.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	streg.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gostop');
		Doc.show('gonext-message');
		restartAt = (new Date()).getTime() + 3000;
		restartTimer = setInterval(streg.countDownToRestart, 200);
	}

	streg.countDownToRestart = function()
	{
		var time = restartAt - (new Date()).getTime();
		if (time < 0)
		{
			Wizard.restartWizard();
		}
		else
		{
			Doc.element('gonext-message').innerHTML = '<i>Wizard will restart automatically in ' + Math.ceil(time/1000) + ' seconds...</i>';
		}
	}
	
	streg.stopAutoRestart = function()
	{
		clearTimeout(restartTimer);
		Doc.hide('gonext-message');
		Doc.hide('gostop');
	}

	
	return streg;
}();

Doc.onLoad(SpecimenTubeReg.initPage);

